netapp_ontap.resources.nvme_subsystem API documentation
Description: Copyright &copy; 2020 NetApp Inc. All rights reserved …

# Module `netapp_ontap.resources.nvme_subsystem`

Copyright © 2020 NetApp Inc. All rights reserved.

## Overview

An NVMe subsystem maintains configuration state and namespace access control for a set of NVMe-connected hosts.
The NVMe subsystem REST API allows you to create, update, delete, and discover NVMe subsystems as well as add and remove NVMe hosts that can access the subsystem and associated namespaces.

## Examples

### Creating an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem()
        resource.svm.name = "svm1"
        resource.name = "subsystem1"
        resource.os_type = "linux"
        resource.post(hydrate=True)
        print(resource)

### Creating an NVMe subsystem with multiple NVMe subsystem hosts

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem()
        resource.svm.name = "svm1"
        resource.name = "subsystem2"
        resource.os_type = "vmware"
        resource.hosts = [
            {"nqn": "nqn.1992-01.example.com:host1"},
            {"nqn": "nqn.1992-01.example.com:host2"},
        ]
        resource.post(hydrate=True)
        print(resource)

### Retrieving all NVMe subsystems

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        print(list(NvmeSubsystem.get_collection()))

Try it out

    [
        NvmeSubsystem(
            {
                "uuid": "acde901a-a379-4a91-9ea6-1b728ed6696f",
                "svm": {"name": "svm1", "uuid": "a009a9e7-4081-b576-7575-ada21efcaf16"},
                "name": "subsystem1",
            }
        ),
        NvmeSubsystem(
            {
                "uuid": "bcde901a-a379-4a91-9ea6-1b728ed6696f",
                "svm": {"name": "svm1", "uuid": "a009a9e7-4081-b576-7575-ada21efcaf16"},
                "name": "subsystem2",
            }
        ),
    ]

### Retrieving all NVMe subsystems with OS type linux

Note that the `os_type` query parameter is used to perform the query.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        print(list(NvmeSubsystem.get_collection(os_type="linux")))

Try it out

    [
        NvmeSubsystem(
            {
                "uuid": "acde901a-a379-4a91-9ea6-1b728ed6696f",
                "os_type": "linux",
                "svm": {"name": "svm1", "uuid": "a009a9e7-4081-b576-7575-ada21efcaf16"},
                "name": "subsystem1",
            }
        )
    ]

### Retrieving a specific NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.get()
        print(resource)

Try it out

    NvmeSubsystem(
        {
            "serial_number": "wtJNKNKD-uPLAAAAAAAD",
            "uuid": "acde901a-a379-4a91-9ea6-1b728ed6696f",
            "os_type": "linux",
            "target_nqn": "nqn.1992-08.com.netapp:sn.d04594ef915b4c73b642169e72e4c0b1:subsystem.subsystem1",
            "svm": {"name": "svm1", "uuid": "a009a9e7-4081-b576-7575-ada21efcaf16"},
            "name": "subsystem1",
            "io_queue": {"default": {"depth": 32, "count": 4}},
        }
    )

### Retrieving the NVMe namespaces mapped to a specific NVMe subsystem

Note that the `fields` query parameter is used to specify the desired properties.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.get(fields="subsystem_maps")
        print(resource)

Try it out

    NvmeSubsystem(
        {
            "uuid": "acde901a-a379-4a91-9ea6-1b728ed6696f",
            "subsystem_maps": [
                {
                    "anagrpid": "00000001h",
                    "nsid": "00000001h",
                    "namespace": {
                        "uuid": "eeaaca23-128d-4a7d-be4a-dc9106705799",
                        "name": "/vol/vol1/namespace1",
                    },
                },
                {
                    "anagrpid": "00000002h",
                    "nsid": "00000002h",
                    "namespace": {
                        "uuid": "feaaca23-83a0-4a7d-beda-dc9106705799",
                        "name": "/vol/vol1/namespace2",
                    },
                },
            ],
            "svm": {"name": "svm1", "uuid": "a009a9e7-4081-b576-7575-ada21efcaf16"},
            "name": "subsystem1",
        }
    )

### Adding a comment about an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.comment = "A brief comment about the subsystem"
        resource.patch()

### Deleting an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.delete()

### Deleting an NVMe subsystem with mapped NVMe namespaces

Normally, deleting an NVMe subsystem that has mapped NVMe namespaces is not allowed. The deletion can be forced using the `allow_delete_while_mapped` query parameter.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.delete(allow_delete_while_mapped=True)

### Delete an NVMe subsystem with NVMe subsystem hosts

Normally, deleting an NVMe subsystem with NVMe subsystem hosts is disallowed. The deletion can be forced using the `allow_delete_with_hosts` query parameter.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystem

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystem(uuid="acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.delete(allow_delete_with_hosts=True)

## An NVMe Subsystem Host

An NVMe subsystem host is a network host provisioned to an NVMe subsystem to access namespaces mapped to that subsystem.

## Examples

### Adding an NVMe subsystem host to an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystemHost

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystemHost("acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.nqn = "nqn.1992-01.com.example:subsys1.host1"
        resource.post(hydrate=True)
        print(resource)

### Adding multiple NVMe subsystem hosts to an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystemHost

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystemHost("acde901a-a379-4a91-9ea6-1b728ed6696f")
        resource.records = [
            {"nqn": "nqn.1992-01.com.example:subsys1.host2"},
            {"nqn": "nqn.1992-01.com.example:subsys1.host3"},
        ]
        resource.post(hydrate=True)
        print(resource)

### Retrieving all NVMe subsystem hosts for an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystemHost

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        print(
            list(NvmeSubsystemHost.get_collection("acde901a-a379-4a91-9ea6-1b728ed6696f"))
        )

Try it out

    [
        NvmeSubsystemHost({"nqn": "nqn.1992-01.com.example:subsys1.host1"}),
        NvmeSubsystemHost({"nqn": "nqn.1992-01.com.example:subsys1.host2"}),
        NvmeSubsystemHost({"nqn": "nqn.1992-01.com.example:subsys1.host3"}),
    ]

### Retrieving a specific NVMe subsystem host for an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystemHost

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystemHost(
            "acde901a-a379-4a91-9ea6-1b728ed6696f",
            nqn="nqn.1992-01.com.example:subsys1.host1",
        )
        resource.get()
        print(resource)

Try it out

    NvmeSubsystemHost(
        {
            "subsystem": {"uuid": "acde901a-a379-4a91-9ea6-1b728ed6696f"},
            "nqn": "nqn.1992-01.com.example:subsys1.host1",
            "io_queue": {"depth": 32, "count": 4},
        }
    )

### Deleting an NVMe subsystem host from an NVMe subsystem

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import NvmeSubsystemHost

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = NvmeSubsystemHost(
            "acde901a-a379-4a91-9ea6-1b728ed6696f",
            nqn="nqn.1992-01.com.example:subsys1.host1",
        )
        resource.delete()

## Classes

`class NvmeSubsystem (*args, **kwargs)`

  An NVMe subsystem maintains configuration state and namespace access control for a set of NVMe-connected hosts.

  Initialize the instance of the resource.

  Any keyword arguments are set on the instance as properties. For example, if the class was named 'MyResource', then this statement would be true:

      MyResource(name='foo').name == 'foo'

  ## Args

  `*args`
    Each positional argument represents a parent key as used in the URL of the object. That is, each value will be used to fill in a segment of the URL which refers to some parent object. The order of these arguments must match the order they are specified in the URL, from left to right.
  `**kwargs`
    each entry will have its key set as an attribute name on the instance and its value will be the value of that attribute.

  ### Ancestors

  - Resource

  ### Static methods

  `def count_collection(*args, connection: HostConnection = None, **kwargs) -> int`

    Retrieves NVMe subsystems.

    ### Related ONTAP commands

    - `vserver nvme subsystem host show`
    - `vserver nvme subsystem map show`
    - `vserver nvme subsystem show`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Fetch a count of all objects of this type from the host.

    This calls GET on the object to determine the number of records. It is more efficient than calling get_collection() because it will not construct any objects. Query parameters can be passed in as kwargs to determine a count of objects that match some filtered criteria.

    ## Args

    `*args`
      Each entry represents a parent key which is used to build the path to the child object. If the URL definition were /api/foos/{foo.name}/bars, then to get the count of bars for a particular foo, the foo.name value should be passed.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host. These query parameters can affect the count. A return_records query param will be ignored.

    ## Returns

    On success, returns an integer count of the objects of this type. On failure, returns -1.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400, or if there is no connection available to use either passed in or on the library.

  `def delete_collection(*args, body: typing.Union = None, connection: HostConnection = None, **kwargs) -> NetAppResponse`

    Removes an NVMe subsystem.

    ### Related ONTAP commands

    - `vserver nvme subsystem delete`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Delete all objects in a collection which match the given query.

    All records on the host which match the query will be deleted.

    ## Args

    `*args`
      Each entry represents a parent key which is used to build the path to the child object. If the URL definition were /api/foos/{foo.name}/bars, then to delete the collection of bars for a particular foo, the foo.name value should be passed.
    `body`
      The body of the delete request. This could be a Resource instance or a dictionary object.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host. Only resources matching this query will be patched.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  `def find(*args, connection: HostConnection = None, **kwargs) -> Resource`

    Retrieves NVMe subsystems.

    ### Related ONTAP commands

    - `vserver nvme subsystem host show`
    - `vserver nvme subsystem map show`
    - `vserver nvme subsystem show`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Find an instance of an object on the host given a query.

    The host will be queried with the provided key/value pairs to find a matching resource. If 0 are found, None will be returned. If more than 1 is found, an error will be raised or returned. If there is exactly 1 matching record, then it will be returned.

    ## Args

    `*args`
      Each entry represents a parent key which is used to build the path to the child object. If the URL definition were /api/foos/{foo.name}/bars, then to find a bar for a particular foo, the foo.name value should be passed.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host.

    ## Returns

    A `Resource` object containing the details of the object or None if no matches were found.

    ## Raises

    `NetAppRestError`: If the API call returned more than 1 matching resource.

  `def get_collection(*args, connection: HostConnection = None, max_records: int = None, **kwargs) -> typing.Iterable`

    Retrieves NVMe subsystems.

    ### Related ONTAP commands

    - `vserver nvme subsystem host show`
    - `vserver nvme subsystem map show`
    - `vserver nvme subsystem show`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Fetch a list of all objects of this type from the host.

    This is a lazy fetch, making API calls only as necessary when the result of this call is iterated over. For instance, if max_records is set to 5, then iterating over the collection causes an API call to be sent to the server once for every 5 records. If the client stops iterating before getting to the 6th record, then no additional API calls are made.

    ## Args

    `*args`
      Each entry represents a parent key which is used to build the path to the child object. If the URL definition were /api/foos/{foo.name}/bars, then to get the collection of bars for a particular foo, the foo.name value should be passed.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `max_records`
      The maximum number of records to return per call
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host.

    ## Returns

    A list of `Resource` objects

    ## Raises

    `NetAppRestError`: If there is no connection available to use either passed in or on the library. This would be not be raised when get_collection() is called, but rather when the result is iterated.

  `def patch_collection(body: dict, *args, connection: HostConnection = None, **kwargs) -> NetAppResponse`

    Updates an NVMe subsystem.

    ### Related ONTAP commands

    - `vserver nvme subsystem modify`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Patch all objects in a collection which match the given query.

    All records on the host which match the query will be patched with the provided body.

    ## Args

    `body`
      A dictionary of name/value pairs to set on all matching members of the collection.
    `*args`
      Each entry represents a parent key which is used to build the path to the child object. If the URL definition were /api/foos/{foo.name}/bars, then to patch the collection of bars for a particular foo, the foo.name value should be passed.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host. Only resources matching this query will be patched.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  ### Methods

  `def delete(self, body: typing.Union = None, poll: bool = True, poll_interval: typing.Union = None, poll_timeout: typing.Union = None, **kwargs) -> NetAppResponse`

    Removes an NVMe subsystem.

    ### Related ONTAP commands

    - `vserver nvme subsystem delete`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Send a deletion request to the host for this object.

    ## Args

    `body`
      The body of the delete request. This could be a Resource instance or a dictionary object.
    `poll`
      If set to True, the call will not return until the asynchronous job on the host has completed. Has no effect if the host did not return a job response.
    `poll_interval`
      If the operation returns a job, this specifies how often to query the job for updates.
    `poll_timeout`
      If the operation returns a job, this specifies how long to continue monitoring the job's status for completion.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  `def get(self, **kwargs) -> NetAppResponse`

    Retrieves an NVMe subsystem.

    ### Expensive properties

    There is an added cost to retrieving values for these properties. They are not included by default in GET results and must be explicitly requested using the `fields` query parameter. See `Requesting specific fields` to learn more. * `subsystem_maps.*`

    ### Related ONTAP commands

    - `vserver nvme subsystem host show`
    - `vserver nvme subsystem map show`
    - `vserver nvme subsystem show`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Fetch the details of the object from the host.

    Requires the keys to be set (if any). After returning, new or changed properties from the host will be set on the instance.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  `def patch(self, hydrate: bool = False, poll: bool = True, poll_interval: typing.Union = None, poll_timeout: typing.Union = None, **kwargs) -> NetAppResponse`

    Updates an NVMe subsystem.

    ### Related ONTAP commands

    - `vserver nvme subsystem modify`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Send the difference in the object's state to the host as a modification request.

    Calculates the difference in the object's state since the last time we interacted with the host and sends this in the request body.

    ## Args

    `hydrate`
      If set to True, after the response is received from the call, a a GET call will be made to refresh all fields of the object.
    `poll`
      If set to True, the call will not return until the asynchronous job on the host has completed. Has no effect if the host did not return a job response.
    `poll_interval`
      If the operation returns a job, this specifies how often to query the job for updates.
    `poll_timeout`
      If the operation returns a job, this specifies how long to continue monitoring the job's status for completion.
    `**kwargs`
      Any key/value pairs passed will normally be sent as query parameters to the host. If any of these pairs are parameters that are sent as formdata then only parameters of that type will be accepted and all others will be discarded.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  `def post(self, hydrate: bool = False, poll: bool = True, poll_interval: typing.Union = None, poll_timeout: typing.Union = None, **kwargs) -> NetAppResponse`

    Creates an NVMe subsystem.

    ### Required properties

    - `svm.uuid` or `svm.name` - Existing SVM in which to create the NVMe subsystem.
    - `name` - Name for NVMe subsystem. Once created, an NVMe subsytem cannot be renamed.
    - `os_type` - Operating system of the NVMe subsystem's hosts.

    ### Related ONTAP commands

    - `vserver nvme subsystem create`

    ### Learn more

    - `DOC /protocols/nvme/subsystems`

    Send this object to the host as a creation request.

    ## Args

    `hydrate`
      If set to True, after the response is received from the call, a a GET call will be made to refresh all fields of the object.
    `poll`
      If set to True, the call will not return until the asynchronous job on the host has completed. Has no effect if the host did not return a job response.
    `poll_interval`
      If the operation returns a job, this specifies how often to query the job for updates.
    `poll_timeout`
      If the operation returns a job, this specifies how long to continue monitoring the job's status for completion.
    `**kwargs`
      Any key/value pairs passed will normally be sent as query parameters to the host. If any of these pairs are parameters that are sent as formdata then only parameters of that type will be accepted and all others will be discarded.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  ### Inherited members

  - `Resource`:
    - `from_dict`
    - `get_collection_url`
    - `get_connection`
    - `instance_location`
    - `path_keys`
    - `poll`
    - `set_connection`
    - `to_dict`
`class NvmeSubsystemSchema (*, only: typing.Union = None, exclude: typing.Union = (), many: bool = False, context: typing.Dict = None, load_only: typing.Union = (), dump_only: typing.Union = (), partial: typing.Union = False, unknown: str = None)`

  The fields of the NvmeSubsystem object

  ### Ancestors

  - netapp_ontap.resource.ResourceSchema
  - marshmallow.schema.Schema
  - marshmallow.base.SchemaABC

  ### Class variables

  `comment GET POST PATCH`

    A configurable comment for the NVMe subsystem. Optional in POST and PATCH.

  `delete_on_unmap GET POST PATCH`

    An option that causes the subsystem to be deleted when the last subsystem map associated with it is deleted. This property defaults to false when the subsystem is created.

  `hosts GET POST`

    The NVMe hosts configured for access to the NVMe subsystem. Optional in POST.

  `io_queue GET POST PATCH`

    The io_queue field of the nvme_subsystem.

  `links GET`

    The links field of the nvme_subsystem.

  `name GET POST`

    The name of the NVMe subsystem. Once created, an NVMe subsystem cannot be renamed. Required in POST.

    Example: subsystem1

  `os_type GET POST`

    The host operating system of the NVMe subsystem's hosts. Required in POST.

    Valid choices:

    - linux
    - vmware
    - windows
  `serial_number GET`

    The serial number of the NVMe subsystem.

    Example: wCVsgFMiuMhVAAAAAAAB

  `subsystem_maps GET`

    The NVMe namespaces mapped to the NVMe subsystem.
    There is an added cost to retrieving property values for `subsystem_maps`. They are not populated for either a collection GET or an instance GET unless explicitly requested using the `fields` query parameter. See `Requesting specific fields` to learn more.

  `svm GET POST PATCH`

    The svm field of the nvme_subsystem.

  `target_nqn GET`

    The NVMe qualified name (NQN) used to identify the NVMe storage target.

    Example: nqn.1992-01.example.com:string

  `uuid GET`

    The unique identifier of the NVMe subsystem.

    Example: 1cd8a442-86d1-11e0-ae1c-123478563412